<?php

/**
 * @file
 * Basic functions for the skeleton module
 */

include_once('skeleton_common.inc');

/**
 * Create the configuration page
 */
function skeleton_admin() {
  $result = db_query("SELECT skeleton_id, skeleton FROM {skeleton} ORDER BY skeleton_id");
  $header = array(t('Id'), t('Name'), array('data' => t('Options'), 'colspan' => 3));
  while ($skeleton = db_fetch_object($result)) {
    $extra = $extra2 = $extra3 = '';
    if (user_access('create new instances')) {
      $extra = l(t('create new instance'), 'admin/content/skeleton/create/'. $skeleton->skeleton_id);
    }
    if (user_access('configure skeleton outlines')) {
      $extra2 = l(t('edit skeleton'), 'admin/content/skeleton/edit/'. $skeleton->skeleton_id);
      $extra3 = l(t('delete skeleton'), 'admin/content/skeleton/delete/'. $skeleton->skeleton_id);
    }  
    $rows[] = array($skeleton->skeleton_id, check_plain($skeleton->skeleton), $extra, $extra2, $extra3);
  }
  if (!empty($rows)) {
    return theme('table', $header, $rows);
  }
  else {
    return t('<p>No skeleton outlines have been created.</p>');
  }
}

/**
 * Add a new skeleton
 */
function skeleton_add() {
  $output = '';
  drupal_set_title(t('Skeleton: Add'));
  $output .= '<h3>'. t('Add New Skeleton') .'</h3>';
  $output .= drupal_get_form('skeleton_add_form');
  return $output;
}

/**
 * FormsAPI for skeleton_add() 
 */
function skeleton_add_form() {
  $form = array();
  $form['skeleton'] = array(
    '#type' => 'fieldset',
    '#title' => t('Skeleton information'),
    '#collapsible' => TRUE
  );
  $form['skeleton']['skeleton'] = array(
    '#type' => 'textfield',
    '#title' => t('Skeleton name'),
    '#description' => t('Enter a unique name to identify this skeleton.'),
    '#required' => TRUE
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Create skeleton')
  );
  return $form;
}

/**
 * FormsAPI for skeleton_add() 
 */
function skeleton_add_form_validate($form_id, $form_values) {
  $check = db_result(db_query("SELECT COUNT(skeleton_id) FROM {skeleton} WHERE skeleton = '%s'", $form_values['skeleton']));
  if ($check > 0) {
    form_set_error('skeleton', t('The name of the skeleton must be unique.'));
  }
}

/**
 * FormsAPI for skeleton_add() 
 */
function skeleton_add_form_submit($form_id, $form_values) {
  db_query("INSERT INTO {skeleton} (skeleton) VALUES ('%s')", $form_values['skeleton']);
  drupal_set_message(t('Skeleton added.'));
  $skeleton = skeleton_get_by_name($form_values['skeleton']);
  drupal_goto('admin/content/skeleton/edit/'. $skeleton->skeleton_id);
}

/**
 * Edit a skeleton definition
 *
 * @param $skeleton
 * An object from the {skeleton} table, indicating the outline to act upon.
 * @return 
 * Themed HTML, including the outline form
 */
function skeleton_edit_instance($skeleton) {
  $output = '';
  drupal_set_title(t('Skeleton: Edit'));
  $output .= '<h3>'. check_plain($skeleton->skeleton) .'</h3>';
  include_once('skeleton_instance.inc');
  return skeleton_define($skeleton);
}

/**
 * Delete a skeleton
 *
 * @param $skeleton
 * An object from the {skeleton} table, indicating the outline to act upon.
 * @return 
 * Themed HTML, including the deletion form 
 */
function skeleton_delete_instance($skeleton) {
  $output = '';
  drupal_set_title(t('Skeleton: Delete'));
  $output .= '<h3>'. check_plain($skeleton->skeleton) .'</h3>';
  $output .= t('<p>Are you sure you want to delete this skeleton outline?</p>');
  $output .= drupal_get_form('skeleton_delete_form', $skeleton->skeleton_id);
  return $output;
}

/**
 * FormsAPI for skeleton_delete_instance()
 */
function skeleton_delete_form($skeleton_id) {
  $form = array();
  $form['skeleton_id'] = array('#type' => 'value', '#value' => $skeleton_id);
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Delete skeleton'),
    '#prefix' => l(t('Cancel action'), 'admin/content/skeleton') .'<br />'
  );
  return $form;
}

/**
  * FormsAPI for skeleton_delete_form()
  */
function skeleton_delete_form_submit($form_id, $form_values) {
  db_query("DELETE from {skeleton} WHERE skeleton_id = %d", $form_values['skeleton_id']);
  // TODO add delete for legacy items
  drupal_set_message(t('Skeleton deleted.'));
  drupal_goto('admin/content/skeleton');
}

/**
 * Create a new instance of a skeleton
 * This function generates the actual book pages.
 *
 * @param $skeleton
 * An object from the {skeleton} table, indicating the outline to act upon.
 * @return 
 * Themed HTML, including the instance creation form  
 */
function skeleton_create_instance($skeleton) {
  $output = '';
  drupal_set_title(t('Skeleton: Create'));
  $output .= '<h3>'. check_plain($skeleton->skeleton) .'</h3>';
  $tree = skeleton_get_tree($skeleton->skeleton_id);
  if (empty($tree)) {
    $output .= t('<p>You must first assign at least one template to this skeleton outline.</p>');
    $output .= '<p>'. l('Edit skeleton', 'admin/content/skeleton/edit/'. $skeleton->skeleton_id) .'</p>';
    return $output;
  }
  $output .= drupal_get_form('skeleton_create_instance_form', $skeleton, $data);
  return $output;
}

/**
 * FormsAPI for skeleton_create_instance()
 */
function skeleton_create_instance_form($skeleton) {
  $form['instance'] = array(
    '#type' => 'fieldset',
    '#title' => t('Test'),
    '#collapsible' => TRUE
  );
  $form['instance']['title'] = array(
    '#type' => 'textfield',
    '#size' => 40,
    '#title' => t('Book title'),
    '#required' => TRUE,
    '#description' => t('The title of the new book you wish to create'),
  );
  $form['instance']['body'] = array(
    '#type' => 'textarea',
    '#cols' => 40,
    '#rows' => 10,
    '#title' => t('Book introduction'),
    '#required' => TRUE,
    '#description' => t('A description of the contents of this book.'),
    '#suffix' => t('<p>After you create this instance, you will be able to edit the complete book page.</p>'),
  );  
  $form['instance']['skeleton_id'] = array(
    '#type' => 'value',
    '#value' => $skeleton->skeleton_id
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => 'Create new book from skeleton'
  );
  return $form;
}

/**
 * FormsAPI for skeleton_create_instance()
 * Creates the nodes from the outline using drupal_execute().
 *
 * Note: The nid fetching sequence is sketchy and might need rewriting
 */
function skeleton_create_instance_form_submit($form_id, $form_values) {
  global $user;
  $nodes = skeleton_get_tree($form_values['skeleton_id']);
  if (!empty($nodes)) {
    $parents = array();
    // get the next node id and use it for the parent
    $parent_node = array('uid' => $user->uid, 'name' => $user->name, 'title' => $form_values['title'], 'type' => 'book', 'body' => $form_values['body'], 'parent' => 0);
    // get the id we just created
    $parent_id = db_result(db_query("SELECT id FROM {sequences} WHERE name = '{node}_nid'")) + 1;
    $goto_id = $parent_id;
    drupal_execute('node_form', $parent_node, $parent_node);
    // iterate through the dummy nodes and create the book pages
    foreach ($nodes as $data) {
      $node = unserialize($data->node_data);
      $parents[$data->template_id] = $nid;
      // if parents are assigned by the skeleton, use them
      if ($data->parent > 0 && isset($parents[$data->parent])) {
        $parent_id = $parents[$data->parent];
      }
      drupal_execute($data->node_type .'_node_form', $node, $node);
      // we have to manually insert the book data -- though hook_form_alter might do it as well
      $nid = db_result(db_query("SELECT id FROM {sequences} WHERE name = '{node}_nid'"));
      $vid = db_result(db_query("SELECT vid FROM {node} WHERE nid = %d", $nid));
      db_query("INSERT INTO {book} (vid, nid, parent, weight) VALUES (%d, %d, %d, %d)", $vid, $nid, $parent_id, $data->weight);
    }
  }  
  else {
    form_set_error(t('This skeleton is empty.'), 'form');
  }
  drupal_goto('node/'. $goto_id . '/edit');
}