<?php

/**
 * @file
 * Functions for defining an instance of a book generated by the skeleton module
 */

include_once('skeleton_common.inc');

/**
 * Show the templates assigned to a skeleton and those still available.
 *
 * @param $skeleton
 * An object from the {skeleton} table, indicating the outline to act upon.
 * @return $output
 * Themed HTML, including the outline form
 */
function skeleton_define($skeleton) {
  $output = '<h3>'. check_plain($skeleton->skeleton) .'</h3>';
  $output .= t('<p>Below are the current templates assigned to this skeleton outline.</p>');
  $output .= t('<h4>Current templates</h4>');
  $templates = skeleton_get_tree($skeleton->skeleton_id);
  $output .= '<p>'. l(t('Create new instance'), 'admin/content/skeleton/create/'. $skeleton->skeleton_id) .'</p>';
  $output .= drupal_get_form('skeleton_define_form', $skeleton, $templates);
  // show the remaining options
  $remove = array();
  foreach ($templates as $i => $template) {
    $remove[] = $template->template_id;
  }  
  if (!empty($remove)) {
    $result = db_query("SELECT template_id, template, node_type FROM {skeleton_template} WHERE template_id NOT IN (". implode(', ', $remove) .") ORDER BY template_id");
  }
  else {
    $result = db_query("SELECT template_id, template, node_type FROM {skeleton_template} ORDER BY template_id");
  }
  $header = array(t('Id'), t('Name'), t('Node type'), array('data' => t('Options'), 'colspan' => 2));
  $rows = array();
  while ($template = db_fetch_object($result)) {
    $link = l(t('Add template'), 'admin/content/skeleton/assign/add/'. $skeleton->skeleton_id .'/'. $template->template_id);
    $rows[] = array($template->template_id, check_plain($template->template), $template->node_type, $link);
  }  
  $output .= t('<h4>Available templates</h4>');
  if (!empty($rows)) {
    $output .= theme('table', $header, $rows);
  }
  else {
    $output .= t('<p>No skeleton templates are available to add to this outline.</p>');
  }  
  $output .= '<p>'. l(t('Create new template'), 'admin/content/skeleton/template/add') .'</p>';
  return $output;
}

/**
 * Callback to add a template to a skeleton
 *
 * @param $action
 * Determines what to do with the template (add / remove).
 * @param $skeleton
 * An object from the {skeleton} table, indicating the outline to act upon
 * @param $template
 * An object from the {skeleton_template} table, indicating the template to act upo 
 */
function skeleton_assign($action, $skeleton, $template) {
  $check = db_result(db_query("SELECT COUNT(*) FROM {skeleton_data} WHERE skeleton_id = %d AND template_id = %d", $skeleton->skeleton_id, $template->template_id));
  if (!$check && $action == 'add') {
    $weight = 0;
    $parent = 0;
    // TODO -- calculate the parent and weight correctly?
    db_query("INSERT INTO {skeleton_data} (skeleton_id, template_id, parent, weight) VALUES (%d, %d, %d, %d)", $skeleton->skeleton_id, $template->template_id, $parent, $weight);
    drupal_set_message(t('Template assigned'));
    drupal_goto('admin/content/skeleton/edit/'. $skeleton->skeleton_id);
  }  
  elseif ($check && $action == 'remove') {
    // update the children with the new parent, keep the original weight
    if ($template->parent) {
      $parent = $template->parent;
    }
    else {
      $parent = 0;
    }  
    db_query("UPDATE {skeleton_data} SET parent = %d WHERE skeleton_id = %d AND parent = %d", $parent, $skeleton->skeleton_id, $template->template_id);
    
    // remove
    db_query("DELETE FROM {skeleton_data} WHERE skeleton_id = %d AND template_id = %d", $skeleton->skeleton_id, $template->template_id);
    drupal_set_message(t('Template removed'));
    drupal_goto('admin/content/skeleton/edit/'. $skeleton->skeleton_id);
  }
  else {
    return 'failed';
  }
}

/**
 * FormsAPI for skeleton_define()
 * @param $skeleton
 * An object from the {skeleton} table, indicating the outline to act upon
 * @param $templates
 * The template tree for this outline, generated by skeleton_get_tree()
 * @return $form
 * A functional $form array.
 */
function skeleton_define_form($skeleton, $templates) {
  $form = array();
  $form['#tree'] = TRUE;
  $form['skeleton_id'] = array('#type' => 'value', '#value' => $skeleton->skeleton_id);  
  foreach ($templates as $i => $template) {
    $options = array(0 => '<none>');
    foreach ($templates as $temp) {
      if ($temp->template_id != $template->template_id && (empty($template->children) || !in_array($temp->template_id, $template->children))) {
        $options[$temp->template_id] = $temp->template;
      }
    }
    $form[$i]['template_id'] = array('#type' =>'value', '#value' => $template->template_id);
    $form[$i]['temp_id'] = array('#value' => $template->template_id);    
    $form[$i]['template'] = array('#value' => str_repeat('-', $template->depth) .' '. check_plain($template->template));
    $form[$i]['node_type'] = array('#value' => check_plain($template->node_type));
    $form[$i]['parent'] = array('#type' => 'select', 
      '#default_value' => $template->parent,
      '#options' => $options,
    );
    $form[$i]['weight'] = array('#type' => 'weight', '#default_value' => $template->weight, '#delta' => 15);    
    $form[$i]['modify'] = array('#value' => l(t('edit'), 'admin/content/skeleton/template/edit/'. $template->template_id));
    $form[$i]['delete'] = array('#value' => l(t('remove'), 'admin/content/skeleton/assign/remove/'. $skeleton->skeleton_id .'/'. $template->template_id));
  }
  $form['notes'] = array('#value' => '<p>'. t('If you change a template parent, all the children of that parent will be reassigned as well.') .'</p>');
  $form['submit'] = array('#type' => 'submit', '#value' => t('Save skeleton outline'));    
  return $form;
}

/**
 * FormsAPI for skeleton_define()
 */
function skeleton_define_form_submit($form_id, $form_values) {
  foreach ($form_values as $key => $template) {
    // only the numeric keys are templates, so process those
    if (is_numeric($key)) {
      db_query("UPDATE {skeleton_data} SET parent = %d, weight = %d WHERE skeleton_id = %d AND template_id = %d", $template['parent'], $template['weight'], $form_values['skeleton_id'], $template['template_id']);
    }
  }
  drupal_set_message(t('Skeleton changes saved.'));
}

/**
 * FormsAPI for skeleton_define()
 * Formats the form in a table.
 */
function theme_skeleton_define_form($form) {
  // Build rows
  $rows = array();
  $last_region = '';
  $last_status = 1;
  foreach (element_children($form) as $i) {
    $template = &$form[$i];
    if (isset($template['template_id'])) {
    $row = array(
      drupal_render($template['temp_id']),    
      drupal_render($template['template']),
      drupal_render($template['node_type']),      
      drupal_render($template['parent']),      
      drupal_render($template['weight']),
    );
    $row[] = drupal_render($template['modify']);
    $row[] = drupal_render($template['delete']);
    $rows[] = $row;
    }
  }
  // Finish table
  $header = array(t('Id'), t('Template'), t('Node type'), t('Parent'), t('Weight'));
  $header[] = array('data' => t('Operations'), 'colspan' => 2);
  $output = theme('table', $header, $rows, array('id' => 'skeleton'));
  $output .= drupal_render($form);
  return $output;  
}