<?php

/**
 * @file
 * Template creation functions for the skeleton module
 */

include_once('skeleton_common.inc');

/**
 * View all templates
 *
 * @param $template_id
 * Unused, sent by the router function
 * @return
 * A table of current templates or an error message.
 */
function skeleton_view_template($template_id) {
  drupal_set_title(t('Skeleton templates'));
  $result = db_query("SELECT * FROM {skeleton_template} ORDER BY template_id");
  $header = array(t('Id'), t('Name'), t('Node type'), array('data' => t('Options'), 'colspan' => 2));
  while ($template = db_fetch_object($result)) {
    $extra = l(t('edit template'), 'admin/content/skeleton/template/edit/'. $template->template_id);
    $extra2 = l(t('delete template'), 'admin/content/skeleton/template/delete/'. $template->template_id);  
    $rows[] = array($template->template_id, check_plain($template->template), $template->node_type, $extra, $extra2);
  }
  if (!empty($rows)) {
    return theme('table', $header, $rows);
  }
  else {
    return t('<p>No skeleton templates have been created.</p>');
  }
}

/**
 * Add new template
 *
 * @param $template_id
 * The id of the template to act upon 
 * @return
 * A themed HTML page, including the form
 */
function skeleton_add_template($template_id) {
  drupal_set_title(t('Skeleton templates: Add'));
  return drupal_get_form('skeleton_add_template_form', $template_id);
}

/**
 * FormsAPI for skeleton_add_template()
 */
function skeleton_add_template_form() {
  $form = array();
  $form['skeleton_template'] = array(
    '#type' => 'fieldset',
    '#title' => t('Template information'),
    '#collapsible' => TRUE
  );
  $form['skeleton_template']['template'] = array(
    '#type' => 'textfield',
    '#title' => t('Template name'),
    '#description' => t('Enter a unique name to identify this template.'),
    '#required' => TRUE
  );
  $types = node_get_types();
  foreach ($types as $type) {
    if (function_exists($type->module .'_form') && node_access('create', $type->type)) { 
      $options[$type->type] = $type->name;
    }  
  }
  $form['skeleton_template']['node_type'] = array(
    '#type' => 'select',
    '#title' => t('Node type'),
    '#options' => $options,
    '#description' => t('Select the node type for this template.'),
    '#required' => TRUE
  );  
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Create template')
  );
  // TODO merge this form with the edit form?
  return $form;
}

/**
 * FormsAPI for skeleton_add_template()
 */
function skeleton_add_template_form_validate($form_id, $form_values) {
  $check = db_result(db_query("SELECT COUNT(template_id) FROM {skeleton_template} WHERE template = '%s'", $form_values['template']));
  if ($check > 0) {
    form_set_error('template', t('The name of the template must be unique.'));
  }
}

/**
 * FormsAPI for skeleton_add_template()
 */
function skeleton_add_template_form_submit($form_id, $form_values) {
  db_query("INSERT INTO {skeleton_template} (template, node_type) VALUES ('%s', '%s')", $form_values['template'], $form_values['node_type']);
  drupal_set_message(t('Template added.'));
  $template = skeleton_get_template_by_name($form_values['template']);
  drupal_goto('admin/content/skeleton/template/edit/'. $template->template_id);
}

/**
 * Delete template
 *
 * @param $template_id
 * The id of the template to act upon 
 * @return
 * A themed HTML page, including the form
 */
function skeleton_delete_template($template_id) {
  drupal_set_title(t('Skeleton templates: Delete'));  
  return drupal_get_form('skeleton_delete_template_form', $template_id);
}

/**
 * FormsAPI for skeleton_delete_template()
 */
function skeleton_delete_template_form($template_id) {
  $form = array();
  $form['template_id'] = array('#type' => 'value', '#value' => $template_id);
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Delete template'),
    '#prefix' => l(t('Cancel action'), 'admin/content/skeleton/template') .'<br />'
  );
  return $form;
}

/**
  * FormsAPI for skeleton_delete_template()
  */
function skeleton_delete_template_form_submit($form_id, $form_values) {
  db_query("DELETE from {skeleton_template} WHERE template_id = %d", $form_values['template_id']);
  drupal_set_message(t('Template deleted.'));
  drupal_goto('admin/content/skeleton/template/view');
}

/**
 * Edit template
 *
 * @param $template_id
 * The id of the template to act upon 
 * @return
 * A themed HTML page, including the form 
 */
function skeleton_edit_template($template_id) {
  drupal_set_title(t('Skeleton templates: Edit'));
  $template = skeleton_get_template($template_id);
  $check = FALSE;
  if (!empty($template->node_data)) {
    $node = $template->node_data;
    $node['type'] = $template->node_type;
    $check = TRUE;
  }
  else {
    $node = array('uid' => $user->uid, 'name' => $user->name, 'type' => $template->node_type);   
  }  
  $node_form = $template->node_type .'_node_form';
  return drupal_get_form($node_form, $node);  
}

/**
 * Implements hook_form_alter()
 *
 * This function lets us use the default node type form to create a template.
 * There may be some issues with its handling of array data, like checkboxes
 * radios, and other odd CCK field types.
 *
 * Note that it does not support file uploads
 */
function skeleton_form_alter($form_id, &$form) {
  // only alter forms handled through the skeleton interface
  $form_type = explode('_', $form_id);
  $form_node = $form_type[0] .'_node_form';
  if ($form_node == $form_id && arg(0) == 'admin' && arg(2) == 'skeleton' && arg(3) == 'template' && arg(4) == 'edit') {
    $template = skeleton_get_template(arg(5));    
    if (!empty($template->node_data)) {
      $node = $template->node_data;    
    }
    // handles some form elements (radios mostly) that don't get set correctly
    foreach ($form as $key => $value) {
      if (isset($node[$key]['key']) && isset($form[$key]['key']['#default_value'])) {
        $form[$key]['key']['#default_value'] = $node[$key]['key'];        
      }  
    }
    // publishing options are handled oddly -- this may affect all checkbox types
    foreach ($form['options'] as $key => $value) {
      if (isset($node[$key])) {
        $form['options'][$key]['#default_value'] = $node[$key];
      }
    }
    // handle taxonomies -- this may apply to all select elements
    foreach ($form['taxonomy'] as $key => $value) {
      if (is_numeric($key)) {
        $form['taxonomy'][$key]['#default_value'] = $node['taxonomy'][$key];
      }
    }
    $form['template_id'] = array('#type' => 'value', '#value' => $template->template_id);
    $form['type'] = array('#type' => 'value', '#value' => $template->node_type);    
    $form['skeleton_template'] = array(
      '#type' => 'fieldset',
      '#title' => t('Template information'),
      '#collapsible' => TRUE,
      '#weight' => -100
    );
    $form['skeleton_template']['template'] = array(
      '#type' => 'textfield',
      '#title' => t('Template name'),
      '#value' => $template->template,
      '#description' => t('The unique name to identify this template.'),
      '#required' => TRUE
    );
    $options = node_get_types('names');
    $form['skeleton_template']['node_type'] = array(
      '#type' => 'select',
      '#title' => t('Node type'),
      '#value' => $template->node_type,    
      '#options' => $options,
      '#description' => t('The node type for this template. <em>This value cannot be edited.</em>'),
      '#disabled' => TRUE    
    );      
    // unset the values we don't want or cannot save
    $unset = array('menu', 'path', 'attachments', 'field_file', 'log', 'preview', 'submit', '#after_build', '#submit');
    foreach ($unset as $item) {
      unset($form[$item]);
    }
    if (module_exists('nodeaccess') && user_access('grant node permissions')) {
      $form['skeleton_template_grants'] = array(
        '#type' => 'fieldset',
        '#title' => t('Node access rules'),
        '#collapsible' => TRUE,
        '#collapsed' => TRUE,
        '#weight' => 100,
        '#theme' => 'skeleton_na_form_element'        
      );      
      // get roles and set default permissions
      $roles = user_roles();
      if (isset($node['rid'])) {
        // set the gid into the array correctly
        $perms = $node['rid'];
        foreach ($roles as $key => $value) {
          $perms[$key] = $node['rid'][$key];
          $perms[$key]['gid'] = $key;
          $perms[$key]['name'] = $roles[$key];
        }
      }
      else {
        $perms = variable_get('nodeaccess_'. $template->node_type, array());
      }  
      foreach ($perms as $perm) {
        $name = $roles[$perm['gid']]; 
        $roles[$perm['gid']] = $perm;
        $roles[$perm['gid']]['name'] = $name;           
      }
      $naform = array();
      if (is_array($roles)) {
        $naform['rid'] = array('#tree' => TRUE);
        $allowed = variable_get('nodeaccess-roles', array());
        foreach ($roles as $key => $field) {
          if ($allowed[$key]) {
            $naform['rid'][$key]['name'] = array('#type' => 'hidden', '#value' => $field['name']);
            $naform['rid'][$key]['grant_view'] = array('#type' => 'checkbox', '#default_value' => $field['grant_view']);
            $naform['rid'][$key]['grant_update'] = array('#type' => 'checkbox', '#default_value' => $field['grant_update']);
            $naform['rid'][$key]['grant_delete'] = array('#type' => 'checkbox', '#default_value' => $field['grant_delete']);
          }
        }
      }     
      $form['skeleton_template_grants']['na'] = $naform;
    }
    $form['#submit']['skeleton_edit_template_form_submit'] = array();
    $form['#validate']['skeleton_edit_template_form_validate'] = array();      
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Update template'),
      '#weight' => 200
    );       
  }
}

/**
 * Theme the nodeaccess part of the form
 */
function theme_skeleton_na_form_element($form) { 
  // role table
  $roles = element_children($form['na']['rid']);
  if (count($roles) > 0) {
    $header = array(t('Role'), t('View'), t('Edit'), t('Delete'));
    foreach ($roles as $key) {
      $row = array();
      $row[] = $form['na']['rid'][$key]['name']['#value'] . drupal_render($form['rid'][$key]['name']);
      $row[] = drupal_render($form['na']['rid'][$key]['grant_view']);
      $row[] = drupal_render($form['na']['rid'][$key]['grant_update']);
      $row[] = drupal_render($form['na']['rid'][$key]['grant_delete']);
      $rows[] = $row;
    }
    $output .= theme('table', $header, $rows);
  } 
  return $output;
}  

/**
 * FormsAPI
 */
function skeleton_edit_template_form_validate($form_id, $form_values) {
  // stub function
  return;
}

/**
 * FormsAPI for handling the template form
 */
function skeleton_edit_template_form_submit($form_id, $form_values) {
  // these items are not stored node data
  $node = $form_values;
  $unset = array('template_id', 'template', 'node_type', 'nid', 'vid', 'created', 'changed', 'date', 'op', 'submit', 'form_token', 'form_id');
  foreach ($unset as $item) {
    unset($node[$item]);
  }
  db_query("UPDATE {skeleton_template} SET node_data = '%s' WHERE template_id = %d", serialize($node), $form_values['template_id']);
  drupal_set_message(t('Template successfully updated.'));
}